desc 'Creates project installable package.'
task :package do |t, args|
  require 'fileutils'
  # Zip is internally required (and modified) by warbler. In case zip
  # is installed as standalone gem (and is not used internal zip from
  # stdlibs), explicit require of zip fails (and this error cannot be
  # rescued because it also breaks warbler itself).
  # require 'zip'
  require 'uu/os/util/version_loader'

  if !File.exists?('target')
    Dir.mkdir('target')
  else
    FileUtils.rm_r(Dir['target/*'])
  end

  puts 'Processing dependencies'
  require 'bundler'
  # Workaround for https://github.com/mkristian/jbundler/issues/59
  begin
    require 'bundler/vendor/thor/lib/thor/actions/create_file'
  rescue LoadError
    # Require is not valid for older versions of bundler
  end
  require 'bundler/cli'
  Bundler::CLI.start(['install'], :debug => false)
  Bundler.reset!

  # Force using of supported JRuby version
  if Gem.loaded_specs['jruby-jars'].nil?
    supported_jruby_version = '1.7.21'
    installed = false
    Gem::Specification.find_all_by_name('jruby-jars') do |spec|
      if spec.version.to_s == supported_jruby_version
        installed = true
        break
      end
    end
    if !installed
      puts "Installing jruby-jars #{supported_jruby_version}"
      Gem.install('jruby-jars', supported_jruby_version)
    end
    puts "Using jruby-jars #{supported_jruby_version}"
    gem 'jruby-jars', supported_jruby_version
  end

  require 'warbler'
  fullname = File.basename(Dir.pwd).downcase
  version = File.join(Dir.pwd, 'app', 'version.rb')
  if !File.exists?(version)
    basename = fullname.split('-')[0]
    version = File.join(Dir.pwd, 'lib', basename, 'version.rb')
  end
  version = UU::OS::Util::VersionLoader.get_version(version)
  if !version.nil?
    war_name = "#{fullname}-#{version}"
  else
    war_name = "#{fullname}"
  end

  Dir.mkdir('target') if !File.exists?('target')
  config = Warbler::Config.new
  # TODO Allow additional directories via command line args
  config.dirs = ['lib', 'app', 'config']
  config.autodeploy_dir = 'target'
  config.jar_name = war_name
  # Following configuration should tell warbler to put
  # parameter "jruby.rack.input.rewindable" into web.xml
  # as context-param.
  # This parameter is very important. Without it JRuby
  # Rack buffers body into memory (and creates tempfile
  # if body exceeds reasonable size).
  config.webxml.jruby.rack.input.rewindable = false
  Warbler::Task.new('buildwar', config)
  Rake::Task['buildwar'].execute

end
